from django.test import TestCase
from django.db import IntegrityError

from tagman.models import Tag, TagGroup
from tagman.tests.models import TestItem, TCI, IgnoreTestItem


class TestTags(TestCase):

    def setUp(self):
        self.group = TagGroup(name="test-group")
        self.group.save()
        self.groupb = TagGroup(name="test-group-b")
        self.groupb.save()
        self.groupc = TagGroup(name="system-group", system=True)
        self.groupc.save()
        self.tag1 = Tag(group=self.group, name="test-tag1")
        self.tag1.save()
        self.tag2 = Tag(group=self.group, name="test-tag2")
        self.tag2.save()
        self.item = TestItem(name="test-item")
        self.item.save()
        self.tags = [self.tag1, self.tag2]

    def test_create_tag(self):
        self.assertIsNotNone(self.tag1.pk)

    def test_create_group(self):
        self.assertIsNotNone(self.group.pk)

    def test_add_tag_to_item(self):
        [self.item.tags.add(tag) for tag in self.tags]
        self.assertTrue([tag for tag in self.item.tags.all()
                        if tag in self.tags])

    def test_tags_in_group(self):
        [self.group.tag_set.add(tag) for tag in self.tags]
        self.assertTrue([tag for tag in self.group.tags_for_group()
                         if tag in self.tags])

    def test_unique_tag_in_group(self):
        tag1_clone = Tag(group=self.group, name=self.tag1.name)
        self.assertRaises(IntegrityError, tag1_clone.save)

        # ... also test to see if we can make a tag with the same name as
        # tag1 but a different group. This must now be allowed.
        tag1_noclone = Tag(group=self.groupb, name=self.tag1.name)
        try:
            tag1_noclone.save()
        except IntegrityError, ie:
            self.fail("Unique constraint on group,name fails: {0}"
                      .format(str(ie)))

    def test_unique_tag_in_system_group(self):
        tag = Tag(group=self.groupc, name="foo")
        tag.save()
        tagb = Tag(group=self.groupc, name="foo")
        self.assertRaises(IntegrityError, tagb.save)

    def test_get_tagged_model_items(self):
        self.item.tags.add(self.tag1)
        model_items =\
            self.tag1.tagged_model_items(model_cls=self.item.__class__)
        self.assertTrue(self.item in model_items)

    def test_get_tag_for_string(self):
        self.group.tag_set.add(self.tag1)
        self.assertTrue(self.tag1 == Tag.tag_for_string(str(self.tag1)))

    def test_get_tags_for_string(self):
        string = ""
        for idx, tag in enumerate(self.tags):
            self.group.tag_set.add(tag)
            string += "%s%s" % (str(tag), ","
                      if idx + 1 is not len(self.tags) else "")
        self.assertTrue([tag for tag in Tag.tags_for_string(string)
                         if tag in self.tags])

    def test_get_all_tag_groups(self):
        [self.item.tags.add(tag) for tag in self.tags]
        [self.group.tag_set.add(tag) for tag in self.tags]
        self.assertTrue([group for group in self.item.all_tag_groups()
                         if group == self.group])

    def test_add_tag_str(self):
        [self.item.add_tag_str("%s:%s" % (self.group.name, tag.name))
         for tag in self.tags]
        self.assertTrue([tag for tag in self.item.tags.all()
                         if tag in self.tags])

    def test_get_tagged_items(self):
        self.item.tags.add(self.tag1)
        item_model_name = self.item.__class__.__name__.lower()
        # Create a model that should be ignored
        ignored_model = IgnoreTestItem(name="ignore_me")
        ignored_model.save()
        ignored_model.tags.add(self.tag1)
        ignored_model_name = ignored_model.__class__.__name__.lower()
        # first check the default call returns both
        items = self.tag1.tagged_items()
        self.assertTrue(self.item in items[item_model_name])
        self.assertTrue(ignored_model in items[ignored_model_name])
        # now ignore the model and check that its not returned
        items = self.tag1.tagged_items(ignore_models=[ignored_model_name])
        self.assertTrue(self.item in items[item_model_name])
        self.assertTrue(ignored_model_name not in items.keys())

    def test_get_unique_item_set(self):
        self.item.tags.add(self.tag1)
        item2 = TestItem(name="test-item-2")
        item2.save()
        item2.tags.add(self.tag1)
        items = self.tag1.unique_item_set()
        self.assertTrue(self.item in items)
        self.assertTrue(item2 in items)


class TestSystemTags(TestCase):
    """
    System tags are designed not to appear in most UI - they are auto-added
    by models so that they can be searched on and have other functionality.
    A custom manager on the Tag prevents these appearing in e.g. admin for
    users when tagging items. But other managers allow access to the whole
    world of tags. Here we test these managers, as well as the rendering of
    system tags.
    """
    def setUp(self):
        self.sys_group = TagGroup(name="system_group", system=True)
        self.sys_group.save()
        self.nonsys_group = TagGroup(name="normal_group", system=False)
        self.nonsys_group.save()

        self.tag_a = Tag(group=self.sys_group, name="tag_a")
        self.tag_b = Tag(group=self.sys_group, name="tag_b")
        self.tag_c = Tag(group=self.nonsys_group, name="tag_c")
        self.tag_d = Tag(group=self.nonsys_group, name="tag_d")
        [getattr(self, "tag_%s" % x).save() for x in "abcd"]

        self.systags = set([self.tag_a, self.tag_b])
        self.nonsystags = set([self.tag_c, self.tag_d])

    def test_manager_public(self):
        nonsystags = set(Tag.public_objects.all())
        self.assertEquals(nonsystags, self.nonsystags)

    def test_manager_systags(self):
        systags = set(Tag.sys_objects.all())
        self.assertEquals(systags, self.systags)

    def test_manager_default(self):
        alltags = set(self.systags)
        alltags.update(self.nonsystags)
        returnedtags = set(Tag.objects.all())
        self.assertEquals(alltags, returnedtags)

    def test_sysgroup_representations(self):
        self.assertEquals(str(self.tag_a), "*system_group:tag_a")

    def test_non_sysgroup_representation(self):
        self.assertEquals(str(self.tag_c), "normal_group:tag_c")

    def test_tag_for_string_system_group(self):
        t = Tag.tag_for_string("*system_group:tag_a")
        self.assertEquals(t, self.tag_a)


class TestSlugification(TestCase):
    def setUp(self):
        self.group = TagGroup(name="normal group", system=False)
        self.group.save()

        self.tag_a = Tag(group=self.group, name="tag a")
        self.tag_b = Tag(group=self.group, name="TAG b")
        self.tag_a.save()
        self.tag_b.save()

    def test_slug_values_a(self):
        self.assertEquals(self.tag_a.slug, "tag-a")

    def test_slug_values_b(self):
        self.assertEquals(self.tag_b.slug, "tag-b")


class TestTaggedContentItem(TestCase):
    """
    Test the features of TaggedContentItem, anything that is tagged
    in some way and which will automatically assign itself tags.
    """
    def setUp(self):
        self.tci = TCI()
        self.tci.save()
        self.tci.associate_auto_tags()
        self.tci.save()

    def test_gen_tag_string(self):
        tag_str = self.tci.self_tag_string
        self.assertEquals(tag_str, '*TCI:tci-slug')

    def test_make_self_tag_name(self):
        name = self.tci._make_self_tag_name()
        self.assertEquals(name, 'tci-slug')

    def test_self_autotag(self):
        """Test method which returns object's own unique auto tag"""
        auto_tag = self.tci.self_auto_tag
        self.assertEquals(auto_tag, Tag.objects.get(slug='tci-slug'))

    def test_auto_tag_creation(self):
        self.tci.associate_auto_tags()
        at = self.tci.auto_tags.all()
        self.assertEquals(len(at), 1)
        tag = at[0]
        self.assertEquals(tag.group.name, 'TCI')
        self.assertEquals(tag.name, 'tci-slug')
        self.assertEquals(tag.system, True)

    def test_get_auto_tagged_items(self):
        self.tci.associate_auto_tags()
        auto_tag = self.tci.auto_tags.all()[0]
        tci_models = auto_tag.tagged_model_items(model_cls=self.tci.__class__)
        self.assertTrue(self.tci in tci_models)

    def test_get_tci_from_auto_tag(self):
        """
        Test the Tag.auto_tagged_model_items method by looking to retrieve
        our TCI based on its auto-tag.
        """
        auto_tag = Tag.objects.get(slug='tci-slug')
        item_set = auto_tag.auto_tagged_model_items(model_cls=TCI)
        self.assertEquals(len(item_set), 1)
        self.assertEquals(item_set.pop(), self.tci)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'TagGroup'
        db.create_table('tagman_taggroup', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=100)),
            ('system', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('tagman', ['TagGroup'])

        # Adding model 'Tag'
        db.create_table('tagman_tag', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=100)),
            ('group', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['tagman.TagGroup'], null=True, blank=True)),
        ))
        db.send_create_signal('tagman', ['Tag'])

        # Adding unique constraint on 'Tag', fields ['id', 'group']
        db.create_unique('tagman_tag', ['id', 'group_id'])


    def backwards(self, orm):
        # Removing unique constraint on 'Tag', fields ['id', 'group']
        db.delete_unique('tagman_tag', ['id', 'group_id'])

        # Deleting model 'TagGroup'
        db.delete_table('tagman_taggroup')

        # Deleting model 'Tag'
        db.delete_table('tagman_tag')


    models = {
        'tagman.tag': {
            'Meta': {'unique_together': "(('id', 'group'),)", 'object_name': 'Tag'},
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['tagman.TagGroup']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'})
        },
        'tagman.taggroup': {
            'Meta': {'object_name': 'TagGroup'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'system': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        }
    }

    complete_apps = ['tagman']



"""Tagman admin classes and also helpers/mixins for users of Tagman"""
from django.contrib import admin
from tagman.models import TagGroup
from tagman.models import Tag
from django import forms


class TaggedContentItemForm(forms.ModelForm):
    """
    Form for use on model admins that have a 'tags' field in which you want
    a nice filtered list without system tags polluting it. Typical for all
    TaggedContentItem models.
    """
    def __init__(self, *args, **kwargs):
        """
        Find all fields in a page ending in 'tags', assume that they are a
        tags M2M and reset the widget's choices to a filtered list that
        excludes system tags.

        This is very crude and rather inelegant but it solved a particular
        problem. It is suggested this is used with care, or used as an
        example of how to manage filtering if you'd like to do some such
        in another way.
        """
        super(TaggedContentItemForm, self).__init__(*args, **kwargs)
        wtf = Tag.objects.filter(group__system=False)
        wlist = [w for t, w in self.fields.items() if t.endswith("tags")]
        choices = []
        for choice in wtf:
            choices.append((choice.id, str(choice)))
        [setattr(w, 'choices', choices) for w in wlist]


class TaggedContentAdminMixin(object):
    """
    When this is the first in the list of base classes for the admin class
    of a model that has tags it will ensure your 'tags' are filtered.
    """
    form = TaggedContentItemForm


class TagGroupAdmin(admin.ModelAdmin):
    list_display = ["name", "slug", "system"]
    search_fields = ["name"]
    list_filter = ["system"]
    prepopulated_fields = {"slug": ("name",)}


class TagAdmin(admin.ModelAdmin):
    list_display = ["name", "slug", "group", "system"]
    search_fields = ["name"]
    list_filter = ["group"]
    prepopulated_fields = {"slug": ("name",)}

    def system(self, _object):
        return _object.system
    system.short_description = u'System'
    system.boolean = True

    def queryset(self, request):
        # use our manager, rather than the default one
        qs = self.model.objects.get_query_set()

        # we need this from the superclass method
        # otherwise we might try to *None, which is bad ;)
        ordering = self.ordering or ()
        if ordering:
            qs = qs.order_by(*ordering)
        return qs

try:
    admin.site.register(Tag, TagAdmin)
    admin.site.register(TagGroup, TagGroupAdmin)
except admin.sites.AlreadyRegistered:
    pass



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Removing unique constraint on 'Tag', fields ['group', 'id']
        db.delete_unique('tagman_tag', ['group_id', 'id'])

        # Adding field 'Tag.archived'
        db.add_column('tagman_tag', 'archived',
                      self.gf('django.db.models.fields.BooleanField')(default=False),
                      keep_default=False)


        # Changing field 'Tag.group'
        db.alter_column('tagman_tag', 'group_id', self.gf('django.db.models.fields.related.ForeignKey')(default=1, to=orm['tagman.TagGroup']))
        # Adding unique constraint on 'Tag', fields ['group', 'name']
        db.create_unique('tagman_tag', ['group_id', 'name'])


    def backwards(self, orm):
        # Removing unique constraint on 'Tag', fields ['group', 'name']
        db.delete_unique('tagman_tag', ['group_id', 'name'])

        # Deleting field 'Tag.archived'
        db.delete_column('tagman_tag', 'archived')


        # Changing field 'Tag.group'
        db.alter_column('tagman_tag', 'group_id', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['tagman.TagGroup'], null=True))
        # Adding unique constraint on 'Tag', fields ['group', 'id']
        db.create_unique('tagman_tag', ['group_id', 'id'])


    models = {
        'tagman.tag': {
            'Meta': {'unique_together': "(('name', 'group'),)", 'object_name': 'Tag'},
            'archived': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['tagman.TagGroup']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'slug': ('django.db.models.fields.SlugField', [], {'default': "''", 'max_length': '100'})
        },
        'tagman.taggroup': {
            'Meta': {'object_name': 'TagGroup'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'slug': ('django.db.models.fields.SlugField', [], {'default': "''", 'max_length': '100'}),
            'system': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        }
    }

    complete_apps = ['tagman']


from django.db import models

from tagman.models import TaggedItem
from tagman.models import TaggedContentItem


class TestItem(TaggedItem):
    name = models.CharField(max_length=100, default="test")

    class Meta:
        app_label = "tagman"

    def __unicode__(self):
        return str(self.name)


class IgnoreTestItem(TaggedItem):
    name = models.CharField(max_length=100, default="test")

    class Meta:
        app_label = "tagman"

    def __unicode__(self):
        return str(self.name)


class TCI(TaggedContentItem):
    class Meta:
        app_label = "tagman"

    slug = "tci-slug"



"""
Tagman seeks to solve the problem where system designers need both categories
and tags but can't quite describe what the differences really are.

A compound construct, the qualified tag, has a name but is assigned to a group.

These models implement this idea.
"""
import logging
import itertools

from django.db import models
from django.core.exceptions import FieldError
from django.template.defaultfilters import slugify

TAG_SEPARATOR = ":"
logger = logging.getLogger()


class TagGroup(models.Model):
    """
    A Tag Group is a logical grouping for tags; e.g. tag group 'flavour' could
    have tags 'cinnamon', 'beef' etc which would be distinguish from another
    tag also named 'beef' but in the category 'meat'.
    """
    name = models.CharField(verbose_name='Name', max_length=100, unique=True)
    slug = models.SlugField(max_length=100, default="")
    system = models.BooleanField(default=False,
                                 help_text="Set True for system groups that "
                                           "should not appear for general use")

    def __unicode__(self):
        prefix = "*" if self.system else ""
        return prefix + self.name

    def tags_for_group(self):
        """
        Return the set of tags that are associated with this group
        """
        return self.tag_set.all()

    def save(self, *args, **kwargs):
        """ assign slug if empty """
        if not self.slug:
            self.slug = slugify(self.name)
        super(TagGroup, self).save(*args, **kwargs)


class TagManager(models.Manager):
    def __init__(self, sys=False, archived=True):
        """
        If sys == True, this will return only system tags. If False,
        the default, will return non-system tags only.
        If archived == False, this will return only non-archived Tags.
        """
        super(TagManager, self).__init__()
        self.system_tags = sys
        self.archived = archived

    def get_query_set(self):
        """
        By default return only those objects that are not flagged as
        'system' tags.
        """
        return super(TagManager, self).get_query_set()\
            .exclude(group__system=not self.system_tags).filter(archived=self.archived)


class Tag(models.Model):
    """
    A Tag has a name and is associated with a tag group. Thus we might
    have flavour:cinnamon where the tag is 'cinnamon' and the tag group
    is 'flavour'.
    """
    name = models.CharField(verbose_name='Name', max_length=100)
    slug = models.SlugField(max_length=100, default="")
    group = models.ForeignKey(TagGroup, verbose_name='Group')
    archived = models.BooleanField(default=False)

    objects = models.Manager()
    sys_objects = TagManager(sys=True)
    public_objects = TagManager(sys=False)
    non_archived = TagManager(sys=False, archived=False)

    def save(self, *args, **kwargs):
        """
        Assign slug if empty
        """
        if not self.slug:
            self.slug = slugify(self.name)
        super(Tag, self).save(*args, **kwargs)

    class Meta:
        unique_together = ("name", "group",)

    def __unicode__(self):
        return "{0}{1}".format(str(self.group)
                               + TAG_SEPARATOR
                               if self.group else "", self.name)

    def __repr__(self):
        return "{0}{1}".format(str(self.group.slug)
                               + TAG_SEPARATOR
                               if self.group else "", self.slug)

    def archive(self):
        """
        Set the archive flag to implement soft-delete
        """
        self.archived=True
        self.save()

    @property
    def system(self):
        return self.group.system

    def models_for_tag(self):
        """
        Return the unique set of model names, all of which have had
        instances tagged with this tag.
        @todo: This is *really* hacky. Can we do it more elegantly?
        """
        models = []
        for attribute in dir(self):
            if attribute[-4:] == '_set':
                # we just want the model name, not the set name
                model_name = attribute.split('_')[0]
                #TODO: check if the list already contains the model name?
                models.append(model_name)
            # return the unique set of model names
        #TODO: what is more efficent?
        return set(models)

    def tagged_model_items(self, model_cls=None, model_name="", limit=None,
                           only_auto=False, filter_dict=None):
        """
        Return a unique set of instances of a given model, the class for
        which is passed into model_cls OR the name for which is passed in
        model_name, that are tagged with this tag.

        If `only_auto`==True then return only auto-tagged instances.
        """
        def _get_models_items(query_set):
            items = []
            try:
                _set = getattr(self, query_set)
            except AttributeError:
                logger.exception("Set {0} not found on tag {1}".format(
                    query_set,
                    self
                ))
            else:
                if filter_dict:
                    try:
                        items = _set.filter(**filter_dict)[:limit]
                    except FieldError, e:
                        logger.exception(
                            "Cannot apply filter {0} to set {1}"
                            .format(
                                filter_dict,
                                query_set
                            )
                        )
                else:
                    items = _set.all()[:limit]
            return set(items)

        if model_cls:
            cls_name = model_cls.__name__.lower()
        else:
            cls_name = model_name.lower()

        model_set = set()
        if not only_auto:
            model_set.update(_get_models_items("{0}_set".format(cls_name)))
        model_set.update(_get_models_items("{0}_auto_tagged_set"
                                           .format(cls_name)))

        return model_set

    def auto_tagged_model_items(self, model_cls=None, model_name="",
                                limit=None):
        """
        Convenience method to return all auto-tagged instances for class
        and tag. See tagged_model_items which this calls with
        only_auto=True
        """
        return self.tagged_model_items(model_cls, model_name, limit,
                                       only_auto=True)

    def tagged_items(self, limit=None, only_auto=False, ignore_models=[],
                     filter_dict=None):
        """
        Return a dictionary, keyed on model name, with each value the
        set of items of that model tagged with this tag.
        """
        models = self.models_for_tag()
        ignore_models = [model.lower() for model in ignore_models]
        rdict = {}
        for model in models:
            if model not in ignore_models:
                rdict[model] = self.tagged_model_items(model_name=model,
                                                       only_auto=only_auto,
                                                       filter_dict=filter_dict,
                                                       limit=limit)
        return rdict

    def unique_item_set(self, limit=None, only_auto=False, ignore_models=[],
                        filter_dict=None):
        """
        Return the unique item set for a tag
        """
        item_set = set()
        tagged_items = self.tagged_items(limit=limit,
                                         only_auto=only_auto,
                                         filter_dict=filter_dict,
                                         ignore_models=ignore_models)
        # merge all tagged items into a unique set
        item_set.update(itertools.chain(*tagged_items.values()))

        return item_set

    @classmethod
    def tag_for_string(cls, s):
        """
        Given a tag representation as "[*]GRP:NAME", return
        the tag instance.

        @todo: handle the [TagGroup|Tag].DoesNotExist exceptions
        """
        s = s.strip('* ')  # representation of system group prefixed *
        groupname, tagname = s.split(TAG_SEPARATOR)
        try:
            grp = TagGroup.objects.get(name=groupname)
            tag = Tag.objects.get(name=tagname, group=grp)
        except TagGroup.DoesNotExist:
            raise Tag.DoesNotExist()
        return tag

    @classmethod
    def get_or_create(cls, group_name, tag_name, system=False):
        """
        Like get_or_create on a manager but driven by distinct strings and
        creates the TagGroup if required.
        """
        group, _ = TagGroup.objects.get_or_create(name=group_name,
                                                  system=system)
        tag, created = Tag.objects.get_or_create(name=tag_name,
                                                 slug=slugify(tag_name),
                                                 group=group)
        if created:
            logger.debug("Created tag via get_or_create "
                         "with repr {0} and ID {1}"
                         .format(repr(tag), tag.id))
        return tag

    @classmethod
    def get_or_create_tag_for_string(cls, s):
        """
        Given a tag representation as "[*]GRP:NAME", return the tag
        instance.
        """
        group, name = s.strip('* ').split(':')
        is_system = True if s.strip()[0] == '*' else False
        return Tag.get_or_create(group, name, is_system)

    @classmethod
    def tags_for_string(cls, s):
        """
        Given a comma delimited list of tag string representations, e.g.::

        <GRP>:<NAME>,<GRP>:<NAME2>...

        return the list of tag instances that these represent.
        @todo: handle case where no tag instance returned
        """
        tokens = s.strip().split(',')
        _tags = [Tag.tag_for_string(t) for t in tokens]
        if not _tags:
            return None
        return _tags


class TaggedItem(models.Model):
    """
    Abstract base class for all models that wish to have tagging. Provides
    `tags` and `auto_tags` fields as well as convenience methods
    for setting and querying.
    """
    tags = models.ManyToManyField(Tag, related_name="%(class)s_set",
                                  blank=True)
    auto_tags = models.ManyToManyField(
        Tag,
        related_name="%(class)s_auto_tagged_set",
        blank=True, editable=False)

    class Meta:
        abstract = True

    def add_tag_str(self, string, auto_tag=False):
        """
        Create a tag from a string and add to tags.

        If auto_tag = True, return from the auto_tags list instead of tags
        """
        tags = self.auto_tags if auto_tag else self.tags
        tag = Tag.get_or_create_tag_for_string(string)
        tags.add(tag)
        return tag

    def all_tag_groups(self, auto_tag=False):
        """
        Return all set of unique tag groups of tags associated with this
        instance. If auto_tag = True, return from the auto_tags list instead
        of tags
        """
        tags = self.auto_tags if auto_tag else self.tags
        return set(tag.group for tag in tags.all())


class TaggedContentItem(TaggedItem):
    """
    Mixin for models that would have features such as auto-tagging
    enabled.
    """
    class Meta:
        abstract = True

    def _make_self_tag_name(self):
        """
        Override this in concrete class to change the slug used, for example
        if the slug field is not called `slug`
        """
        return self.slug

    @property
    def self_tag_string(self):
        """
        Generate the string representation for the auto tag that this
        should be assigned. e.g. return `*<classname>:<tag name>`
        """
        tag_group = self.__class__.__name__
        tag_name = self._make_self_tag_name()
        return "*{0}:{1}".format(tag_group, tag_name)

    @property
    def self_auto_tag(self):
        """
        Return the tag instance that is this object's own auto tag
        """
        tag_str = self.self_tag_string
        my_tag = [t for t in self.auto_tags.all() if str(t) == tag_str]
        if not my_tag:
            raise Exception("{0} has yet to be auto-tagged".format(self))
        return my_tag[0]

    def associate_auto_tags(self):
        """
        Automatically tag myself (by adding to auto_tags):
        *<model name>:<slug>.

        Override _make_self_tag_name(self) to change the slug used.

        Typically this method would be called on an instance in a post-save
        signal handler for a model.
        """
        tag = self.add_tag_str(self.self_tag_string, auto_tag=True)
        logger.info("Auto tagging {0} with {1}".format(str(self), repr(tag)))
        return tag



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Removing unique constraint on 'Tag', fields ['name']
        db.delete_unique('tagman_tag', ['name'])


    def backwards(self, orm):
        # Adding unique constraint on 'Tag', fields ['name']
        db.create_unique('tagman_tag', ['name'])


    models = {
        'tagman.tag': {
            'Meta': {'unique_together': "(('id', 'group'),)", 'object_name': 'Tag'},
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['tagman.TagGroup']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'slug': ('django.db.models.fields.SlugField', [], {'default': "''", 'max_length': '100'})
        },
        'tagman.taggroup': {
            'Meta': {'object_name': 'TagGroup'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'slug': ('django.db.models.fields.SlugField', [], {'default': "''", 'max_length': '100'}),
            'system': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        }
    }

    complete_apps = ['tagman']


#!/usr/bin/env python
from setuptools import setup, find_packages

setup(
    name = "tagman",
    version = "0.1.11",
    author = "ReThought Ltd",
    author_email = "code@rethought-solutions.com",
    url = "https://github.com/Rethought/tagman.git",

    packages = find_packages('src'),
    package_dir = {'':'src'},
    license = "BSD",
    keywords = "django, tagging, tagman",
    description = "Curated tagging app for Django",
    classifiers = [
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ]
)
